@@ -2,7 +2,7 @@ import h5py
 from multiprocessing import Pool
 import numpy as np
 import pandas as pd
-from astropy.table import Table, vstack
+from astropy.table import Table, vstack, QTable
 import tables
 from tables import open_file
 import os
@@ -16,12 +16,10 @@ from eventio import Histograms
 from eventio.search_utils import yield_toplevel_of_type
 from .lstcontainers import ThrownEventsHistogram, ExtraMCInfo, MetaData
 from tqdm import tqdm
-#from ctapipe.tools.stage1 import Stage1ProcessorTool
-from astropy.utils import deprecated
+# from ctapipe.tools.stage1 import Stage1ProcessorTool
 from ctapipe.instrument import OpticsDescription, CameraGeometry, CameraDescription, CameraReadout, \
     TelescopeDescription, SubarrayDescription
 from pyirf.simulations import SimulatedEventsInfo
-from astropy import table
 
 import logging
 
@@ -34,7 +32,6 @@ __all__ = [
     'get_dataset_keys',
     'write_simtel_energy_histogram',
     'write_mcheader',
-    'write_array_info',
     'check_thrown_events_histogram',
     'check_mcheader',
     'check_metadata',
@@ -48,20 +45,20 @@ __all__ = [
     'write_dataframe',
     'write_dl2_dataframe',
     'write_calibration_data',
-    'read_dl2_to_pyirf',
+    'read_mc_dl2_to_QTable',
+    'read_data_dl2_to_QTable',
     'read_dl2_params',
     'extract_observation_time',
     'merge_dl2_runs'
 ]
 
-
-dl1_params_tel_mon_ped_key = 'dl1/event/telescope/monitoring/pedestal'
-dl1_params_tel_mon_cal_key = '/dl1/event/telescope/monitoring/calibration'
-dl1_params_lstcam_key = 'dl1/event/telescope/parameters/LST_LSTCam'
-dl1_images_lstcam_key = 'dl1/event/telescope/image/LST_LSTCam'
-dl2_params_lstcam_key = 'dl2/event/telescope/parameters/LST_LSTCam'
-dl1_params_src_dep_lstcam_key = 'dl1/event/telescope/parameters_src_dependent/LST_LSTCam'
-dl2_params_src_dep_lstcam_key = 'dl2/event/telescope/parameters_src_dependent/LST_LSTCam'
+dl1_params_tel_mon_ped_key = "dl1/event/telescope/monitoring/pedestal"
+dl1_params_tel_mon_cal_key = "/dl1/event/telescope/monitoring/calibration"
+dl1_params_lstcam_key = "dl1/event/telescope/parameters/LST_LSTCam"
+dl1_images_lstcam_key = "dl1/event/telescope/image/LST_LSTCam"
+dl2_params_lstcam_key = "dl2/event/telescope/parameters/LST_LSTCam"
+dl1_params_src_dep_lstcam_key = "dl1/event/telescope/parameters_src_dependent/LST_LSTCam"
+dl2_params_src_dep_lstcam_key = "dl2/event/telescope/parameters_src_dependent/LST_LSTCam"
 
 HDF5_ZSTD_FILTERS = tables.Filters(
     complevel=5,            # enable compression, 5 is a good tradeoff between compression and speed
@@ -81,7 +78,7 @@ def read_simu_info_hdf5(filename):
     """
 
     with HDF5TableReader(filename) as reader:
-        mc_reader = reader.read('/simulation/run_config', SimulationConfigContainer())
+        mc_reader = reader.read("/simulation/run_config", SimulationConfigContainer())
         mc = next(mc_reader)
 
     return mc
@@ -104,19 +101,24 @@ def read_simu_info_merged_hdf5(filename):
 
     """
     with open_file(filename) as file:
-        simu_info = file.root['simulation/run_config']
+        simu_info = file.root["simulation/run_config"]
         colnames = simu_info.colnames
-        skip = {'num_showers', 'shower_prog_start', 'detector_prog_start', 'obs_id'}
+        skip = {"num_showers", "shower_prog_start", "detector_prog_start", "obs_id"}
         for k in filter(lambda k: k not in skip, colnames):
             assert np.all(simu_info[:][k] == simu_info[0][k])
-        num_showers = simu_info[:]['num_showers'].sum()
+        num_showers = simu_info[:]["num_showers"].sum()
 
     combined_mcheader = read_simu_info_hdf5(filename)
-    combined_mcheader['num_showers'] = num_showers
+    combined_mcheader["num_showers"] = num_showers
 
     for k in combined_mcheader.keys():
-        if combined_mcheader[k] is not None and combined_mcheader.fields[k].unit is not None:
-            combined_mcheader[k] = u.Quantity(combined_mcheader[k], combined_mcheader.fields[k].unit)
+        if (
+            combined_mcheader[k] is not None
+            and combined_mcheader.fields[k].unit is not None
+        ):
+            combined_mcheader[k] = u.Quantity(
+                combined_mcheader[k], combined_mcheader.fields[k].unit
+            )
 
     return combined_mcheader
 
@@ -134,11 +136,12 @@ def get_dataset_keys(filename):
     list of keys
     """
     dataset_keys = []
+
     def walk(name, obj):
         if type(obj) == h5py._hl.dataset.Dataset:
             dataset_keys.append(name)
 
-    with h5py.File(filename, 'r') as file:
+    with h5py.File(filename, "r") as file:
         file.visititems(walk)
 
     return dataset_keys
@@ -168,7 +171,7 @@ def get_stacked_table(filenames_list, node):
     return vstack(table_list)
 
 
-def stack_tables_h5files(filenames_list, output_filename='merged.h5', keys=None):
+def stack_tables_h5files(filenames_list, output_filename="merged.h5", keys=None):
     """
     In theory similar to auto_merge_h5files but slower. Keeping it for reference.
     Merge h5 files produced by lstchain using astropy.
@@ -189,7 +192,6 @@ def stack_tables_h5files(filenames_list, output_filename='merged.h5', keys=None)
         merged_table.write(output_filename, path=k, append=True)
 
 
-
 def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None, merge_arrays=False, filters=HDF5_ZSTD_FILTERS):
     """
     Automatic merge of HDF5 files.
@@ -200,6 +202,8 @@ def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None,
     file_list: list of path
     output_filename: path
     nodes_keys: list of path
+    merge_arrays: bool
+    filters
     """
 
     if nodes_keys is None:
@@ -238,14 +242,16 @@ def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None,
             common_keys = keys.intersection(get_dataset_keys(filename))
             with open_file(filename) as file:
                 for k in common_keys:
+                    in_node = file.root[k]
+                    out_node = merge_file.root[k]
                     try:
-                        if merge_arrays:
-                            merge_file.root[k].append(file.root[k].read())
-                        else:
-                            if type(file.root[k]) == tables.table.Table:
-                                merge_file.root[k].append(file.root[k].read())
+                        if isinstance(in_node, tables.table.Table) or merge_arrays:
+                            # doing `.astype(out_node.dtype)` fixes an issue
+                            # when dtypes do not exactly match but are convertible
+                            # https://github.com/cta-observatory/cta-lstchain/issues/671
+                            out_node.append(in_node.read().astype(out_node.dtype))
                     except:
-                        print("Can't append node {} from file {}".format(k, filename))
+                        log.exception("Can't append node {} from file {}".format(k, filename))
             bar.update(1)
 
 
@@ -253,9 +259,9 @@ def merging_check(file_list):
     """
     Check that a list of hdf5 files are compatible for merging regarding:
      - array info
-     - MC simu info
-     - MC histograms
      - metadata
+     - MC simu info (only for simulations)
+     - MC histograms (only for simulations)
 
     Parameters
     ----------
@@ -268,29 +274,38 @@ def merging_check(file_list):
     assert len(file_list) > 1, "The list of files is too short"
     mergeable_list = file_list.copy()
 
-    filename0 = mergeable_list[0]
-    array_info0 = read_array_info(filename0)
-    mcheader0 = read_simu_info_hdf5(filename0)
-    thrown_events_hist0 = read_simtel_energy_histogram(filename0)
-    metadata0 = read_metadata(filename0)
+    first_file = mergeable_list[0]
+    subarray_info0 = SubarrayDescription.from_hdf(first_file)
+    metadata0 = read_metadata(first_file)
+
+    if subarray_info0.name == "MonteCarloArray":
+        mcheader0 = read_simu_info_hdf5(first_file)
+        thrown_events_hist0 = read_simtel_energy_histogram(first_file)
+
     for filename in mergeable_list[1:]:
         try:
-            mcheader = read_simu_info_hdf5(filename)
-            thrown_events_hist = read_simtel_energy_histogram(filename)
             metadata = read_metadata(filename)
             check_metadata(metadata0, metadata)
-            check_mcheader(mcheader0, mcheader)
-            check_thrown_events_histogram(thrown_events_hist0, thrown_events_hist)
-            for ii, table in read_array_info(filename).items():
-                assert (table == array_info0[ii]).all()
-        except:
-            log.exception(f"{filename} cannot be smart merged ¯\_(ツ)_/¯")
+            subarray_info = SubarrayDescription.from_hdf(filename)
+
+            if subarray_info0.name == "MonteCarloArray":
+                mcheader = read_simu_info_hdf5(filename)
+                thrown_events_hist = read_simtel_energy_histogram(filename)
+                check_mcheader(mcheader0, mcheader)
+                check_thrown_events_histogram(thrown_events_hist0, thrown_events_hist)
+
+            assert subarray_info == subarray_info0
+
+        except AssertionError:
+            log.exception(f"{filename} cannot be smart merged '¯\_(ツ)_/¯'")
             mergeable_list.remove(filename)
 
     return mergeable_list
 
 
-def smart_merge_h5files(file_list, output_filename='merged.h5', node_keys=None, merge_arrays=False):
+def smart_merge_h5files(
+    file_list, output_filename="merged.h5", node_keys=None, merge_arrays=False
+):
     """
     Check that HDF5 files are compatible for merging and merge them
 
@@ -298,9 +313,13 @@ def smart_merge_h5files(file_list, output_filename='merged.h5', node_keys=None,
     ----------
     file_list: list of paths to hdf5 files
     output_filename: path to the merged file
+    node_keys
+    merge_arrays: bool
     """
     smart_list = merging_check(file_list)
-    auto_merge_h5files(smart_list, output_filename, nodes_keys=node_keys, merge_arrays=merge_arrays)
+    auto_merge_h5files(
+        smart_list, output_filename, nodes_keys=node_keys, merge_arrays=merge_arrays
+    )
 
     # Merge metadata
     metadata0 = read_metadata(smart_list[0])
@@ -322,14 +341,16 @@ def write_simtel_energy_histogram(source, output_filename, obs_id=None, filters=
     obs_id: float, int, str or None
     """
     # Writing histograms
-    with HDF5TableWriter(filename=output_filename, group_name="simulation", mode="a", filters=filters) as writer:
+    with HDF5TableWriter(
+        filename=output_filename, group_name="simulation", mode="a", filters=filters
+    ) as writer:
         writer.meta = metadata
         for hist in yield_toplevel_of_type(source.file_, Histograms):
             pass
         # find histogram id 6 (thrown energy)
         thrown = None
         for hist in source.file_.histograms:
-            if hist['id'] == 6:
+            if hist["id"] == 6:
                 thrown = hist
 
         thrown_hist = ThrownEventsHistogram()
@@ -337,7 +358,7 @@ def write_simtel_energy_histogram(source, output_filename, obs_id=None, filters=
         thrown_hist.obs_id = obs_id
         if metadata is not None:
             add_global_metadata(thrown_hist, metadata)
-        writer.write('thrown_event_distribution', [thrown_hist])
+        writer.write("thrown_event_distribution", [thrown_hist])
 
 
 def read_simtel_energy_histogram(filename):
@@ -353,7 +374,9 @@ def read_simtel_energy_histogram(filename):
     `lstchain.io.lstcontainers.ThrownEventsHistogram`
     """
     with HDF5TableReader(filename=filename) as reader:
-        histtab = reader.read('/simulation/thrown_event_distribution', ThrownEventsHistogram())
+        histtab = reader.read(
+            "/simulation/thrown_event_distribution", ThrownEventsHistogram()
+        )
         hist = next(histtab)
     return hist
 
@@ -365,155 +388,21 @@ def write_mcheader(mcheader, output_filename, obs_id=None, filters=HDF5_ZSTD_FIL
     Parameters
     ----------
     output_filename: str
-    event: `ctapipe.io.ArrayEventContainer`
     """
 
     extramc = ExtraMCInfo()
-    extramc.prefix = ''  # get rid of the prefix
+    extramc.prefix = ""  # get rid of the prefix
     if metadata is not None:
         add_global_metadata(mcheader, metadata)
         add_global_metadata(extramc, metadata)
 
-    with HDF5TableWriter(filename=output_filename, group_name="simulation", mode="a", filters=filters) as writer:
+    with HDF5TableWriter(
+        filename=output_filename, group_name="simulation", mode="a", filters=filters
+    ) as writer:
         extramc.obs_id = obs_id
         writer.write("run_config", [extramc, mcheader])
 
 
-@deprecated('09/07/2020', message='this function will disappear in lstchain v0.7')
-def write_array_info_08(subarray, output_filename):
-    """
-    Write the array info to a ctapipe v0.8 compatible DL1 HDF5 file
-    This is a temporary solution until we move to ctapipe v0.9.1.
-
-    Parameters
-    ----------
-    subarray: `ctapipe.instrument.subarray.SubarrayDescription`
-    output_filename: str
-    """
-
-    serialize_meta = True
-
-    subarray.to_table().write(
-      output_filename,
-      path="/configuration/instrument/subarray/layout",
-      serialize_meta=serialize_meta,
-      append=True,
-    )
-
-    subarray.to_table(kind="optics").write(
-      output_filename,
-      path="/configuration/instrument/telescope/optics",
-      append=True,
-      serialize_meta=serialize_meta,
-    )
-
-    for telescope_type in subarray.telescope_types:
-      ids = set(subarray.get_tel_ids_for_type(telescope_type))
-      if len(ids) > 0: # only write if there is a telescope with this camera
-        tel_id = list(ids)[0]
-        camera = subarray.tel[tel_id].camera
-        camera_name = f'geometry_{camera}'
-        with tables.open_file(output_filename, mode='a') as f:
-          telescope_chidren = f.root['/configuration/instrument/telescope']._v_children.keys()
-          if 'camera' in telescope_chidren:
-            cameras_name = f.root['/configuration/instrument/telescope/camera']._v_children.keys()
-            if camera_name in cameras_name:
-              print(
-                f'WARNING during lstchain.io.write_array_info_08():',
-                f'camera {camera_name} seems to be already present in the h5 file.'
-              )
-              continue
-        camera.geometry.to_table().write(
-          output_filename,
-          path=f"/configuration/instrument/telescope/camera/geometry_{camera}",
-          append=True,
-          serialize_meta=serialize_meta
-        )
-        camera.readout.to_table().write(
-          output_filename,
-          path=f"/configuration/instrument/telescope/camera/readout_{camera}",
-          append=True,
-          serialize_meta=serialize_meta
-        )
-
-
-@deprecated('09/07/2020', message='this function will disappear in lstchain v0.7')
-def write_array_info(subarray, output_filename):
-    """
-    Write the array info to a HDF5 file
-        - layout info is writen in '/instrument/subarray/layout'
-        - optics info is writen in '/instrument/telescope/optics'
-        - camera info is writen in '/instrument/telescope/camera/{camera}' for each camera in the array
-
-    Parameters
-    ----------
-    subarray: `ctapipe.instrument.subarray.SubarrayDescription`
-    output_filename: str
-    """
-
-    serialize_meta = True
-
-    subarray.to_table().write(
-        output_filename,
-        path="/instrument/subarray/layout",
-        serialize_meta=serialize_meta,
-        append=True
-    )
-
-    subarray.to_table(kind='optics').write(
-        output_filename,
-        path='/instrument/telescope/optics',
-        append=True,
-        serialize_meta=serialize_meta
-    )
-    for telescope_type in subarray.telescope_types:
-        ids = set(subarray.get_tel_ids_for_type(telescope_type))
-        if len(ids) > 0:  # only write if there is a telescope with this camera
-            tel_id = list(ids)[0]
-            camera = subarray.tel[tel_id].camera
-            camera_name = str(camera)
-
-            with tables.open_file(output_filename, mode='a') as f:
-                telescope_chidren = f.root['instrument/telescope']._v_children.keys()
-                if 'camera' in telescope_chidren:
-                    cameras_name = f.root['instrument/telescope/camera']._v_children.keys()
-                    if camera_name in cameras_name:
-                        print(
-                            f'WARNING during lstchain.io.write_array_info():',
-                            f'camera {camera_name} seems to be already present in the h5 file.'
-                        )
-                        continue
-
-            camera.geometry.to_table().write(
-                output_filename,
-                path=f'/instrument/telescope/camera/{camera_name}',
-                append=True,
-                serialize_meta=serialize_meta,
-            )
-
-@deprecated('09/07/2020', message='will be removed in lstchain v0.7')
-def read_array_info(filename):
-    """
-    Read array information from HDF5 file.
-
-    Parameters
-    ----------
-    filename: path
-
-    Returns
-    -------
-    dict
-    """
-    array_info = dict()
-    with open_file(filename) as file:
-        array_info['layout'] = Table(file.root['/instrument/subarray/layout'].read())
-        array_info['optics'] = Table(file.root['/instrument/telescope/optics'].read())
-        for camera in file.root['/instrument/telescope/camera/']:
-            if type(camera) is tables.table.Table:
-                array_info[camera.name] = Table(camera.read())
-    return array_info
-
-
 def read_single_optics(filename, telescope_name):
     """
     Read a specific telescope optics from a DL1 file
@@ -528,15 +417,19 @@ def read_single_optics(filename, telescope_name):
     `ctapipe.instrument.optics.OpticsDescription`
     """
     from astropy.units import Quantity
+
     telescope_optics_path = "/configuration/instrument/telescope/optics"
     telescope_optic_table = Table.read(filename, path=telescope_optics_path)
-    row = telescope_optic_table[np.where(telescope_name == telescope_optic_table['name'])[0][0]]
+    row = telescope_optic_table[
+        np.where(telescope_name == telescope_optic_table["name"])[0][0]
+    ]
     optics_description = OpticsDescription(
-        name=row['name'],
-        num_mirrors=row['num_mirrors'],
-        equivalent_focal_length=row['equivalent_focal_length'] * telescope_optic_table['equivalent_focal_length'].unit,
-        mirror_area=row['mirror_area'] * telescope_optic_table['mirror_area'].unit,
-        num_mirror_tiles=Quantity(row['num_mirror_tiles']),
+        name=row["name"],
+        num_mirrors=row["num_mirrors"],
+        equivalent_focal_length=row["equivalent_focal_length"]
+        * telescope_optic_table["equivalent_focal_length"].unit,
+        mirror_area=row["mirror_area"] * telescope_optic_table["mirror_area"].unit,
+        num_mirror_tiles=Quantity(row["num_mirror_tiles"]),
     )
     return optics_description
 
@@ -551,12 +444,12 @@ def read_optics(filename):
 
     Returns
     -------
-    dictionnary of ctapipe.instrument.optics.OpticsDescription by telescope names
+    dictionary of ctapipe.instrument.optics.OpticsDescription by telescope names
     """
     telescope_optics_path = "/configuration/instrument/telescope/optics"
     telescope_optics_table = Table.read(filename, path=telescope_optics_path)
     optics_dict = {}
-    for telescope_name in telescope_optics_table['name']:
+    for telescope_name in telescope_optics_table["name"]:
         optics_dict[telescope_name] = read_single_optics(filename, telescope_name)
     return optics_dict
 
@@ -574,8 +467,12 @@ def read_single_camera_geometry(filename, camera_name):
     -------
     `ctapipe.instrument.camera.geometry.CameraGeometry`
     """
-    camera_geometry_path = f"/configuration/instrument/telescope/camera/geometry_{camera_name}"
-    camera_geometry = CameraGeometry.from_table(Table.read(filename, camera_geometry_path))
+    camera_geometry_path = (
+        f"/configuration/instrument/telescope/camera/geometry_{camera_name}"
+    )
+    camera_geometry = CameraGeometry.from_table(
+        Table.read(filename, camera_geometry_path)
+    )
     return camera_geometry
 
 
@@ -589,11 +486,13 @@ def read_camera_geometries(filename):
 
     Returns
     -------
-    dictionnary of `ctapipe.instrument.camera.geometry.CameraGeometry` by camera name
+    dictionary of `ctapipe.instrument.camera.geometry.CameraGeometry` by camera name
     """
-    subarray_layout_path = 'configuration/instrument/subarray/layout'
+    subarray_layout_path = "configuration/instrument/subarray/layout"
     camera_geoms = {}
-    for camera_name in set(Table.read(filename, path=subarray_layout_path)['camera_type']):
+    for camera_name in set(
+        Table.read(filename, path=subarray_layout_path)["camera_type"]
+    ):
         camera_geoms[camera_name] = read_single_camera_geometry(filename, camera_name)
     return camera_geoms
 
@@ -611,7 +510,9 @@ def read_single_camera_readout(filename, camera_name):
     -------
     `ctapipe.instrument.camera.readout.CameraReadout`
     """
-    camera_readout_path = f"/configuration/instrument/telescope/camera/readout_{camera_name}"
+    camera_readout_path = (
+        f"/configuration/instrument/telescope/camera/readout_{camera_name}"
+    )
     return CameraReadout.from_table(Table.read(filename, path=camera_readout_path))
 
 
@@ -627,11 +528,13 @@ def read_camera_readouts(filename):
     -------
     dict of `ctapipe.instrument.camera.description.CameraDescription` by tel_id
     """
-    subarray_layout_path = 'configuration/instrument/subarray/layout'
+    subarray_layout_path = "configuration/instrument/subarray/layout"
     camera_readouts = {}
     for row in Table.read(filename, path=subarray_layout_path):
-        camera_name = row['camera_type']
-        camera_readouts[row['tel_id']] = read_single_camera_readout(filename, camera_name)
+        camera_name = row["camera_type"]
+        camera_readouts[row["tel_id"]] = read_single_camera_readout(
+            filename, camera_name
+        )
     return camera_readouts
 
 
@@ -653,7 +556,9 @@ def read_single_camera_description(filename, camera_name):
     return CameraDescription(camera_name, geometry=geom, readout=readout)
 
 
-def read_single_telescope_description(filename, telescope_name, telescope_type, camera_name):
+def read_single_telescope_description(
+    filename, telescope_name, telescope_type, camera_name
+):
     """
     Read a specific telescope description from a DL1 file
 
@@ -669,7 +574,9 @@ def read_single_telescope_description(filename, telescope_name, telescope_type,
     """
     optics = read_single_optics(filename, telescope_name)
     camera_descr = read_single_camera_description(filename, camera_name)
-    return TelescopeDescription(telescope_name, telescope_type, optics=optics, camera=camera_descr)
+    return TelescopeDescription(
+        telescope_name, telescope_type, optics=optics, camera=camera_descr
+    )
 
 
 def read_subarray_table(filename):
@@ -684,7 +591,7 @@ def read_subarray_table(filename):
     -------
     `astropy.table.table.Table`
     """
-    subarray_layout_path = 'configuration/instrument/subarray/layout'
+    subarray_layout_path = "configuration/instrument/subarray/layout"
     return Table.read(filename, path=subarray_layout_path)
 
 
@@ -703,11 +610,13 @@ def read_telescopes_descriptions(filename):
     subarray_table = read_subarray_table(filename)
     descriptions = {}
     for row in subarray_table:
-        tel_name = row['name']
-        camera_type = row['camera_type']
+        tel_name = row["name"]
+        camera_type = row["camera_type"]
         optics = read_single_optics(filename, tel_name)
         camera = read_single_camera_description(filename, camera_type)
-        descriptions[row['tel_id']] = TelescopeDescription(row['name'], row['type'], optics=optics, camera=camera)
+        descriptions[row["tel_id"]] = TelescopeDescription(
+            row["name"], row["type"], optics=optics, camera=camera
+        )
     return descriptions
 
 
@@ -721,23 +630,26 @@ def read_telescopes_positions(filename):
 
     Returns
     -------
-    dictionnary of telescopes positions by tel_id
+    dictionary of telescopes positions by tel_id
     """
     subarray_table = read_subarray_table(filename)
     pos_dict = {}
-    pos_unit = subarray_table['pos_x'].unit
+    pos_unit = subarray_table["pos_x"].unit
     for row in subarray_table:
-        pos_dict[row['tel_id']] = np.array([row['pos_x'], row['pos_y'], row['pos_z']]) * pos_unit
+        pos_dict[row["tel_id"]] = (
+            np.array([row["pos_x"], row["pos_y"], row["pos_z"]]) * pos_unit
+        )
     return pos_dict
 
 
-def read_subarray_description(filename, subarray_name='LST-1'):
+def read_subarray_description(filename, subarray_name="LST-1"):
     """
     Read subarray description from an HDF5 DL1 file
 
     Parameters
     ----------
     filename: str
+    subarray_name : str
 
     Returns
     -------
@@ -745,7 +657,9 @@ def read_subarray_description(filename, subarray_name='LST-1'):
     """
     tel_pos = read_telescopes_positions(filename)
     tel_descrp = read_telescopes_descriptions(filename)
-    return SubarrayDescription(subarray_name, tel_positions=tel_pos, tel_descriptions=tel_descrp)
+    return SubarrayDescription(
+        subarray_name, tel_positions=tel_pos, tel_descriptions=tel_descrp
+    )
 
 
 def check_mcheader(mcheader1, mcheader2):
@@ -765,13 +679,17 @@ def check_mcheader(mcheader1, mcheader2):
     # It does not matter that the number of simulated showers is the same
     keys = list(mcheader1.keys())
     """keys that don't need to be checked: """
-    for k in ['num_showers', 'shower_reuse', 'detector_prog_start', 'detector_prog_id', 'shower_prog_id',
-              'shower_prog_start',
-              ]:
+    for k in [
+        "num_showers",
+        "shower_reuse",
+        "detector_prog_start",
+        "detector_prog_id",
+        "shower_prog_id",
+        "shower_prog_start",
+    ]:
         if k in keys:
             keys.remove(k)
 
-
     for k in keys:
         assert mcheader1[k] == mcheader2[k]
 
@@ -787,7 +705,7 @@ def check_thrown_events_histogram(thrown_events_hist1, thrown_events_hist2):
     """
     assert thrown_events_hist1.keys() == thrown_events_hist2.keys()
     # It does not matter that the number of simulated showers is the same
-    keys = ['bins_energy', 'bins_core_dist']
+    keys = ["bins_energy", "bins_core_dist"]
     for k in keys:
         assert (thrown_events_hist1[k] == thrown_events_hist2[k]).all()
 
@@ -803,7 +721,7 @@ def write_metadata(metadata, output_filename):
     """
     # One cannot write strings with ctapipe HDF5Writer and Tables can write only fixed length string
     # So this metadata is written in the file attributes
-    with open_file(output_filename, mode='a') as file:
+    with open_file(output_filename, mode="a") as file:
         for k, item in metadata.as_dict().items():
             if k in file.root._v_attrs and type(file.root._v_attrs) is list:
                 attribute = file.root._v_attrs[k].extend(metadata[k])
@@ -812,7 +730,6 @@ def write_metadata(metadata, output_filename):
                 file.root._v_attrs[k] = metadata[k]
 
 
-
 def read_metadata(filename):
     """
     Read metadata from a HDF5 file
@@ -842,7 +759,7 @@ def check_metadata(metadata1, metadata2):
     metadata2: `lstchain.io.MetaData`
     """
     assert metadata1.keys() == metadata2.keys()
-    keys = ['LSTCHAIN_VERSION']
+    keys = ["LSTCHAIN_VERSION"]
     for k in keys:
         assert metadata1[k] == metadata2[k]
 
@@ -858,7 +775,7 @@ def global_metadata(source):
     metadata = MetaData()
     metadata.LSTCHAIN_VERSION = lstchain.__version__
     metadata.CTAPIPE_VERSION = ctapipe.__version__
-    metadata.CONTACT = 'LST Consortium'
+    metadata.CONTACT = "LST Consortium"
     metadata.SOURCE_FILENAMES.append(os.path.basename(source.input_url))
 
     return metadata
@@ -893,11 +810,10 @@ def write_subarray_tables(writer, event, metadata=None):
         add_global_metadata(event.simulation, metadata)
         add_global_metadata(event.trigger, metadata)
 
-    writer.write(table_name="subarray/mc_shower", containers=[event.index, event.simulation])
     writer.write(table_name="subarray/trigger", containers=[event.index, event.trigger])
 
 
-def write_dataframe(dataframe, outfile, table_path, mode='a', index=False):
+def write_dataframe(dataframe, outfile, table_path, mode="a", index=False):
     """
     Write a pandas dataframe to a HDF5 file using pytables formatting.
 
@@ -908,11 +824,11 @@ def write_dataframe(dataframe, outfile, table_path, mode='a', index=False):
     table_path: str
         path to the table to write in the HDF5 file
     """
-    if not table_path.startswith('/'):
-        table_path = '/' + table_path
+    if not table_path.startswith("/"):
+        table_path = "/" + table_path
 
     with tables.open_file(outfile, mode=mode) as f:
-        path, table_name = table_path.rsplit('/', maxsplit=1)
+        path, table_name = table_path.rsplit("/", maxsplit=1)
 
         f.create_table(
             path,
@@ -954,7 +870,9 @@ def add_column_table(table, ColClass, col_label, values):
     d[col_label] = ColClass()  # add column
 
     # Step 2: Create new temporary table:
-    newtable = tables.Table(table._v_file.root, '_temp_table', d, filters=table.filters)  # new table
+    newtable = tables.Table(
+        table._v_file.root, "_temp_table", d, filters=table.filters
+    )  # new table
     table.attrs._f_copy(newtable)  # copy attributes
     # Copy table rows, also add new column values:
     for row, value in zip(table, values):
@@ -970,7 +888,6 @@ def add_column_table(table, ColClass, col_label, values):
     return newtable
 
 
-
 def recursive_copy_node(src_file, dir_file, path):
     """
     Copy a node recursively from a src file to a dir file without copying the tables/arrays in the node
@@ -985,19 +902,22 @@ def recursive_copy_node(src_file, dir_file, path):
     path_split = path.split('/')
     while '' in path_split:
         path_split.remove('')
-    assert len(path_split)>0
+    assert len(path_split) > 0
     src_file.copy_node('/',
                        name=path_split[0],
                        newparent=dir_file.root,
                        newname=path_split[0],
                        recursive=False)
     if len(path_split) > 1:
-        recursive_path = os.path.join('/', path_split[0])
+        recursive_path = os.path.join("/", path_split[0])
         for p in path_split[1:]:
-            src_file.copy_node(recursive_path,
-                               name=p,
-                               newparent=dir_file.root[recursive_path],
-                               newname=p, recursive=False)
+            src_file.copy_node(
+                recursive_path,
+                name=p,
+                newparent=dir_file.root[recursive_path],
+                newname=p,
+                recursive=False,
+            )
             recursive_path = os.path.join(recursive_path, p)
 
 
@@ -1007,7 +927,6 @@ def write_calibration_data(writer, mon_index, mon_event, new_ped=False, new_ff=F
     mon_event.calibration.prefix = ''
     mon_index.prefix = ''
 
-
     # update index
     if new_ped:
         mon_index.pedestal_id += 1
@@ -1016,71 +935,76 @@ def write_calibration_data(writer, mon_index, mon_event, new_ped=False, new_ff=F
         mon_index.flatfield_id += 1
         mon_index.calibration_id += 1
 
-
     if new_ped:
         # write ped container
         writer.write(
-            table_name=f'telescope/monitoring/pedestal',
-            containers=[mon_index, mon_event.pedestal]
+            table_name="telescope/monitoring/pedestal",
+            containers=[mon_index, mon_event.pedestal],
         )
 
     if new_ff:
         # write calibration container
         writer.write(
             table_name="telescope/monitoring/flatfield",
-            containers=[mon_index, mon_event.flatfield]
+            containers=[mon_index, mon_event.flatfield],
         )
 
         # write ff container
         writer.write(
             table_name="telescope/monitoring/calibration",
-            containers=[mon_index, mon_event.calibration]
+            containers=[mon_index, mon_event.calibration],
         )
 
 
-def read_dl2_to_pyirf(filename):
+def read_mc_dl2_to_QTable(filename):
     """
-    Read DL2 files from lstchain and convert into pyirf internal format
+    Read MC DL2 files from lstchain and convert into pyirf internal format
+    - astropy.table.QTable
+
     Parameters
     ----------
     filename: path
+
     Returns
     -------
     `astropy.table.QTable`, `pyirf.simulations.SimulatedEventsInfo`
     """
 
-    ## mapping
+    # mapping
     name_mapping = {
-        'mc_energy': 'true_energy',
-        'mc_alt': 'true_alt',
-        'mc_az': 'true_az',
-        'mc_alt_tel': 'pointing_alt',
-        'mc_az_tel': 'pointing_az',
-        'gammaness': 'gh_score',
+        "mc_energy": "true_energy",
+        "mc_alt": "true_alt",
+        "mc_az": "true_az",
+        "mc_alt_tel": "pointing_alt",
+        "mc_az_tel": "pointing_az",
+        "gammaness": "gh_score",
     }
 
     unit_mapping = {
-        'true_energy': u.TeV,
-        'reco_energy': u.TeV,
-        'pointing_alt': u.rad,
-        'pointing_az': u.rad,
-        'true_alt': u.rad,
-        'true_az': u.rad,
-        'reco_alt': u.rad,
-        'reco_az': u.rad,
+        "true_energy": u.TeV,
+        "reco_energy": u.TeV,
+        "pointing_alt": u.rad,
+        "pointing_az": u.rad,
+        "true_alt": u.rad,
+        "true_az": u.rad,
+        "reco_alt": u.rad,
+        "reco_az": u.rad,
     }
 
     simu_info = read_simu_info_merged_hdf5(filename)
-    pyirf_simu_info = SimulatedEventsInfo(n_showers=simu_info.num_showers * simu_info.shower_reuse,
-                                          energy_min=simu_info.energy_range_min,
-                                          energy_max=simu_info.energy_range_max,
-                                          max_impact=simu_info.max_scatter_range,
-                                          spectral_index=simu_info.spectral_index,
-                                          viewcone=simu_info.max_viewcone_radius,
-                                          )
+    pyirf_simu_info = SimulatedEventsInfo(
+        n_showers=simu_info.num_showers * simu_info.shower_reuse,
+        energy_min=simu_info.energy_range_min,
+        energy_max=simu_info.energy_range_max,
+        max_impact=simu_info.max_scatter_range,
+        spectral_index=simu_info.spectral_index,
+        viewcone=simu_info.max_viewcone_radius,
+    )
 
-    events = pd.read_hdf(filename, key=dl2_params_lstcam_key).rename(columns=name_mapping)
-    events = table.QTable.from_pandas(events)
+    events = pd.read_hdf(filename, key=dl2_params_lstcam_key).rename(
+        columns=name_mapping
+    )
+    events = QTable.from_pandas(events)
 
     for k, v in unit_mapping.items():
         events[k] *= v
@@ -1088,8 +1012,46 @@ def read_dl2_to_pyirf(filename):
     return events, pyirf_simu_info
 
 
+def read_data_dl2_to_QTable(filename):
+    """
+    Read data DL2 files from lstchain and return QTable format
+    Parameters
+    ----------
+    filename: path to the lstchain DL2 file
+
+    Returns
+    -------
+    `astropy.table.QTable`
+    """
+
+    # Mapping
+    name_mapping = {
+        "gammaness": "gh_score",
+        "alt_tel": "pointing_alt",
+        "az_tel": "pointing_az",
+    }
+    unit_mapping = {
+        "reco_energy": u.TeV,
+        "pointing_alt": u.rad,
+        "pointing_az": u.rad,
+        "reco_alt": u.rad,
+        "reco_az": u.rad,
+        "dragon_time": u.s,
+    }
+
+    data = pd.read_hdf(filename, key=dl2_params_lstcam_key).rename(columns=name_mapping)
+
+    data = QTable.from_pandas(data)
+
+    # Make the columns as Quantity
+    for k, v in unit_mapping.items():
+        data[k] *= v
+
+    return data
+
+
 def read_dl2_params(t_filename, columns_to_read=None):
-    '''
+    """
     Read specified parameters from a file with DL2 data
 
     Parameters
@@ -1100,7 +1062,7 @@ def read_dl2_params(t_filename, columns_to_read=None):
     Returns
     -------
     Pandas dataframe with DL2 data
-    '''
+    """
     if columns_to_read is not None:
         return pd.read_hdf(t_filename, key=dl2_params_lstcam_key)[columns_to_read]
     else:
@@ -1108,7 +1070,7 @@ def read_dl2_params(t_filename, columns_to_read=None):
 
 
 def extract_observation_time(t_df):
-    '''
+    """
     Calculate observation time
 
     Parameters
@@ -1118,7 +1080,7 @@ def extract_observation_time(t_df):
     Returns
     -------
     Observation duration in seconds
-    '''
+    """
     return pd.to_datetime(t_df.dragon_time.iat[len(t_df)-1], unit='s') -\
            pd.to_datetime(t_df.dragon_time.iat[0], unit='s')
 
@@ -1131,6 +1093,7 @@ def merge_dl2_runs(data_tag, runs, columns_to_read=None, n_process=4):
     ----------
     data_tag: lstchain version tag
     runs: List of run numbers
+    columns_to_read
     n_process: Number of parallel read processes to use
 
     Returns
@@ -1139,16 +1102,18 @@ def merge_dl2_runs(data_tag, runs, columns_to_read=None, n_process=4):
     """
     from functools import partial
     from glob import glob
-    filepath_glob = glob(f'/fefs/aswg/data/real/DL2/*/{data_tag}/*') # Current format of LST data path
+    filepath_glob = glob(f'/fefs/aswg/data/real/DL2/*/{data_tag}/*')  # Current format of LST data path
 
     pool = Pool(n_process)
     filelist = []
     # Create a list of files with matching run numbers
     for filename in filepath_glob:
-        if any(f'Run{run:05}' in filename for run in runs):
+        if any(f"Run{run:05}" in filename for run in runs):
             filelist.append(filename)
 
-    df_list = pool.map(partial(read_dl2_params, columns_to_read=columns_to_read), filelist)
+    df_list = pool.map(
+        partial(read_dl2_params, columns_to_read=columns_to_read), filelist
+    )
 
     observation_times = pool.map(extract_observation_time, df_list)
 
