@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
-""" 
-Script to create pedestal file for low level calibration. 
+"""
+Script to create pedestal file for low level calibration.
 
 To set start sample in waveform --start_r0_waveform i (default i = 11)
 not to use deltaT correction add --deltaT False
@@ -12,28 +12,24 @@ not to use deltaT correction add --deltaT False
 
 Usage:
 
-$> python lstchain_data_create_pedestal_file.py 
---input-file LST-1.1.Run00097.0000.fits.fz 
---output_file drs4_pedestalRun2028.0000.fits 
+$> python lstchain_data_create_pedestal_file.py
+--input-file LST-1.1.Run00097.0000.fits.fz
+--output_file drs4_pedestalRun2028.0000.fits
 --max_events 9000
 
 """
 
-
+from traitlets.config import Config
 import argparse
 import numpy as np
 from astropy.io import fits
-from numba import prange
 
-from ctapipe.io import event_source
-from ctapipe.io import EventSeeker
+from ctapipe.io import EventSource
+
 from distutils.util import strtobool
-from traitlets.config.loader import Config
-from lstchain.calib.camera.r0 import LSTR0Corrections
 from lstchain.calib.camera.drs4 import DragonPedestal
 
 
-
 parser = argparse.ArgumentParser()
 
 # Required arguments
@@ -67,38 +63,29 @@ parser.add_argument('--deltaT', '-s',
 
 args = parser.parse_args()
 
-
+source_config = {
+    "LSTEventSource": {
+        "max_events":args.max_events,
+    }
+}
 def main():
     print("--> Input file: {}".format(args.input_file))
     print("--> Number of events: {}".format(args.max_events))
-    reader = event_source(input_url=args.input_file, max_events=args.max_events)
+
+    reader = EventSource(input_url=args.input_file, config=Config(source_config))
     print("--> Number of files", reader.multi_file.num_inputs())
 
-    seeker = EventSeeker(reader)
-    ev = seeker[0]
-    tel_id = ev.r0.tels_with_data[0]
-    n_modules = ev.lst.tel[tel_id].svc.num_modules
+    for i, event in enumerate(reader):
+        for tel_id in event.trigger.tels_with_trigger:
 
-    config = Config({
-        "LSTR0Corrections": {
-            "tel_id": tel_id
-        }
-    })
-    lst_r0 = LSTR0Corrections(config=config)
+            if i==0:
+                n_modules = event.lst.tel[tel_id].svc.num_modules
+                pedestal = DragonPedestal(tel_id=tel_id, n_module=n_modules, r0_sample_start=args.start_r0_waveform)
 
-    pedestal = DragonPedestal(tel_id=tel_id, n_module=n_modules, r0_sample_start=args.start_r0_waveform)
+            if args.deltaT:
+                reader.r0_r1_calibrator.update_first_capacitors(event)
+                reader.r0_r1_calibrator.time_lapse_corr(event, tel_id)
 
-    if args.deltaT:
-        print("DeltaT correction active")
-        for i, event in enumerate(reader):
-            for tel_id in event.r0.tels_with_data:
-                lst_r0.time_lapse_corr(event, tel_id)
-                pedestal.fill_pedestal_event(event)
-                if i%500 == 0:
-                    print("i = {}, ev id = {}".format(i, event.index.event_id))
-    else:
-        print("DeltaT correction no active")
-        for i, event in enumerate(reader):
             pedestal.fill_pedestal_event(event)
             if i%500 == 0:
                 print("i = {}, ev id = {}".format(i, event.index.event_id))
@@ -106,13 +93,17 @@ def main():
     # Finalize pedestal and write to fits file
     pedestal.finalize_pedestal()
 
-    primaryhdu = fits.PrimaryHDU(ev.lst.tel[tel_id].svc.pixel_ids)
-    secondhdu = fits.ImageHDU(np.int16(pedestal.meanped))
-
-    hdulist = fits.HDUList([primaryhdu, secondhdu])
+    expected_pixel_id = fits.PrimaryHDU(event.lst.tel[tel_id].svc.pixel_ids)
+    pedestal_array = fits.ImageHDU(np.int16(pedestal.meanped),
+                                   name="pedestal array")
+    failing_pixels_column = fits.Column(name='failing pixels',
+                                        array=pedestal.failing_pixels_array,
+                                        format='K')
+    failing_pixels = fits.BinTableHDU.from_columns([failing_pixels_column],
+                                                    name="failing pixels")
+    hdulist = fits.HDUList([expected_pixel_id, pedestal_array, failing_pixels])
     hdulist.writeto(args.output_file)
 
 
 if __name__ == '__main__':
     main()
-
