@@ -19,24 +19,20 @@ from distutils.util import strtobool
 import astropy.units as u
 import numpy as np
 import tables
-from astropy.table import Table
-from ctapipe.containers import HillasParametersContainer
 from ctapipe.image import hillas_parameters
 from ctapipe.image.cleaning import tailcuts_clean, apply_time_delta_cleaning
 from ctapipe.image.morphology import number_of_islands
-from ctapipe.instrument import CameraGeometry, OpticsDescription
+from ctapipe.instrument import SubarrayDescription
 
+from lstchain.calib.camera.pixel_threshold_estimation import get_threshold_from_dl1_file
 from lstchain.io import get_dataset_keys, auto_merge_h5files
+from lstchain.io.config import get_cleaning_parameters
 from lstchain.io.config import get_standard_config
 from lstchain.io.config import read_configuration_file, replace_config
 from lstchain.io.io import dl1_params_lstcam_key, dl1_images_lstcam_key
 from lstchain.io.lstcontainers import DL1ParametersContainer
-from lstchain.io.config import get_cleaning_parameters
-from lstchain.calib.camera.pixel_threshold_estimation import get_threshold_from_dl1_file
 from lstchain.reco.disp import disp
 
-import sys
-
 log = logging.getLogger(__name__)
 
 parser = argparse.ArgumentParser(
@@ -87,7 +83,7 @@ def main():
         config = std_config
 
     if args.pedestal_cleaning:
-        print("Pedestal cleaning")
+        log.info("Pedestal cleaning")
         clean_method_name = 'tailcuts_clean_with_pedestal_threshold'
         sigma = config[clean_method_name]['sigma']
         pedestal_thresh = get_threshold_from_dl1_file(args.input_file, sigma)
@@ -112,9 +108,10 @@ def main():
     if "delta_time" in config[clean_method_name]:
         delta_time = config[clean_method_name]["delta_time"]
 
-    foclen = OpticsDescription.from_name('LST').equivalent_focal_length
-    cam_table = Table.read(args.input_file, path="instrument/telescope/camera/LSTCam")
-    camera_geom = CameraGeometry.from_table(cam_table)
+    subarray_info = SubarrayDescription.from_hdf(args.input_file)
+    tel_id = config["allowed_tels"][0] if "allowed_tels" in config else 1
+    focal_length = subarray_info.tel[tel_id].optics.equivalent_focal_length
+    camera_geom = subarray_info.tel[tel_id].camera.geometry
 
     dl1_container = DL1ParametersContainer()
     parameters_to_update = [
@@ -212,8 +209,8 @@ def main():
                         dl1_container.n_islands = num_islands
                         dl1_container.wl = dl1_container.width / dl1_container.length
                         dl1_container.n_pixels = n_pixels
-                        width = np.rad2deg(np.arctan2(dl1_container.width, foclen))
-                        length = np.rad2deg(np.arctan2(dl1_container.length, foclen))
+                        width = np.rad2deg(np.arctan2(dl1_container.width, focal_length))
+                        length = np.rad2deg(np.arctan2(dl1_container.length, focal_length))
                         dl1_container.width = width
                         dl1_container.length = length
                         dl1_container.log_intensity = np.log10(dl1_container.intensity)
